@@ -7,6 +7,7 @@ from .ormsgpack import (
     OPT_PASSTHROUGH_BIG_INT,
     OPT_PASSTHROUGH_DATACLASS,
     OPT_PASSTHROUGH_DATETIME,
+    OPT_PASSTHROUGH_ENUM,
     OPT_PASSTHROUGH_SUBCLASS,
     OPT_PASSTHROUGH_TUPLE,
     OPT_PASSTHROUGH_UUID,
@@ -35,6 +36,7 @@ __all__ = (
     "OPT_PASSTHROUGH_BIG_INT",
     "OPT_PASSTHROUGH_DATACLASS",
     "OPT_PASSTHROUGH_DATETIME",
+    "OPT_PASSTHROUGH_ENUM",
     "OPT_PASSTHROUGH_SUBCLASS",
     "OPT_PASSTHROUGH_TUPLE",
     "OPT_PASSTHROUGH_UUID",
