@@ -4,11 +4,11 @@ use crate::deserialize::cache::*;
 use crate::deserialize::DeserializeError;
 use crate::exc::*;
 use crate::ffi::*;
+use crate::msgpack::Marker;
 use crate::opt::*;
 use crate::typeref::*;
 use crate::unicode::*;
 use byteorder::{BigEndian, ReadBytesExt};
-use rmp::Marker;
 use simdutf8::basic::{from_utf8, Utf8Error};
 use std::borrow::Cow;
 use std::os::raw::c_char;
