@@ -1,5 +1,5 @@
 use pyo3::ffi::*;
-use std::os::raw::{c_char, c_int, c_uint, c_void};
+use std::os::raw::{c_int, c_uint, c_void};
 use std::ptr::null_mut;
 
 #[repr(C)]
@@ -18,7 +18,7 @@ unsafe extern "C" fn ext_new(
     if Py_SIZE(args) != 2 || !kwds.is_null() {
         PyErr_SetString(
             PyExc_TypeError,
-            "Ext.__new__() takes 2 positional arguments\0".as_ptr() as *const c_char,
+            c"Ext.__new__() takes 2 positional arguments".as_ptr(),
         );
         return null_mut();
     }
@@ -26,7 +26,7 @@ unsafe extern "C" fn ext_new(
     if PyLong_Check(tag) == 0 {
         PyErr_SetString(
             PyExc_TypeError,
-            "Ext.__new__() first argument must be int\0".as_ptr() as *const c_char,
+            c"Ext.__new__() first argument must be int".as_ptr(),
         );
         return null_mut();
     }
@@ -34,7 +34,7 @@ unsafe extern "C" fn ext_new(
     if PyBytes_Check(data) == 0 {
         PyErr_SetString(
             PyExc_TypeError,
-            "Ext.__new__() second argument must be bytes\0".as_ptr() as *const c_char,
+            c"Ext.__new__() second argument must be bytes".as_ptr(),
         );
         return null_mut();
     }
@@ -69,7 +69,7 @@ pub unsafe fn create_ext_type() -> *mut PyTypeObject {
         },
     ];
     let mut spec = PyType_Spec {
-        name: "ormsgpack.Ext\0".as_ptr() as *const c_char,
+        name: c"ormsgpack.Ext".as_ptr(),
         basicsize: std::mem::size_of::<PyExt>() as c_int,
         itemsize: 0,
         flags: Py_TPFLAGS_DEFAULT as c_uint,
