@@ -3,7 +3,6 @@
 use crate::exc::*;
 use crate::ffi::PyDictIter;
 use crate::opt::*;
-use crate::serialize::serializer::pyobject_to_obtype;
 use crate::serialize::serializer::*;
 use crate::typeref::*;
 use crate::unicode::*;
@@ -114,7 +113,7 @@ impl Serialize for DictWithStrKeys {
         let len = ffi!(Py_SIZE(self.ptr)) as usize;
         let mut map = serializer.serialize_map(Some(len)).unwrap();
         for (key, value) in PyDictIter::from_pyobject(self.ptr) {
-            if unlikely!(!is_type!(ob_type!(key.as_ptr()), STR_TYPE)) {
+            if unlikely!(!py_is!(ob_type!(key.as_ptr()), STR_TYPE)) {
                 err!(KEY_MUST_BE_STR)
             }
             let data = unicode_to_str(key.as_ptr());
@@ -171,7 +170,7 @@ impl Serialize for DictWithSortedStrKeys {
         let mut items: SmallVec<[(&str, *mut pyo3::ffi::PyObject); 8]> =
             SmallVec::with_capacity(len);
         for (key, value) in PyDictIter::from_pyobject(self.ptr) {
-            if unlikely!(!is_type!(ob_type!(key.as_ptr()), STR_TYPE)) {
+            if unlikely!(!py_is!(ob_type!(key.as_ptr()), STR_TYPE)) {
                 err!(KEY_MUST_BE_STR)
             }
             let data = unicode_to_str(key.as_ptr());
@@ -235,7 +234,7 @@ impl Serialize for DictWithNonStrKeys {
         let len = ffi!(Py_SIZE(self.ptr)) as usize;
         let mut map = serializer.serialize_map(Some(len)).unwrap();
         for (key, value) in PyDictIter::from_pyobject(self.ptr) {
-            if is_type!(ob_type!(key.as_ptr()), STR_TYPE) {
+            if py_is!(ob_type!(key.as_ptr()), STR_TYPE) {
                 let data = unicode_to_str(key.as_ptr());
                 if unlikely!(data.is_none()) {
                     err!(INVALID_STR)
@@ -251,26 +250,8 @@ impl Serialize for DictWithNonStrKeys {
                     ),
                 )?;
             } else {
-                match pyobject_to_obtype(key.as_ptr(), opts) {
-                    ObType::NumpyScalar
-                    | ObType::NumpyArray
-                    | ObType::Dict
-                    | ObType::List
-                    | ObType::Dataclass
-                    | ObType::Pydantic
-                    | ObType::Unknown => {
-                        err!("Dict key must a type serializable with OPT_NON_STR_KEYS")
-                    }
-                    _ => (),
-                }
                 map.serialize_entry(
-                    &PyObject::new(
-                        key.as_ptr(),
-                        opts,
-                        self.default_calls,
-                        self.recursion + 1,
-                        self.default,
-                    ),
+                    &DictKey::new(key.as_ptr(), opts, self.recursion + 1),
                     &PyObject::new(
                         value.as_ptr(),
                         self.opts,
