@@ -1,6 +1,6 @@
 use crate::opt::*;
 use crate::serialize::datetimelike::NaiveDateTime;
-use crate::typeref::{load_numpy_types, ARRAY_STRUCT_STR, DESCR_STR, DTYPE_STR, NUMPY_TYPES};
+use crate::typeref::{ARRAY_STRUCT_STR, DESCR_STR, DTYPE_STR};
 use chrono::{DateTime, NaiveDate};
 use pyo3::ffi::*;
 use serde::ser::{Serialize, SerializeSeq, Serializer};
@@ -12,38 +12,6 @@ macro_rules! slice {
     };
 }
 
-pub fn is_numpy_scalar(ob_type: *mut PyTypeObject) -> bool {
-    let numpy_types = unsafe { NUMPY_TYPES.get_or_init(load_numpy_types) };
-    if numpy_types.is_none() {
-        false
-    } else {
-        let scalar_types = unsafe { numpy_types.unwrap().as_ref() };
-        ob_type == scalar_types.float64
-            || ob_type == scalar_types.float32
-            || ob_type == scalar_types.float16
-            || ob_type == scalar_types.int64
-            || ob_type == scalar_types.int32
-            || ob_type == scalar_types.int16
-            || ob_type == scalar_types.int8
-            || ob_type == scalar_types.uint64
-            || ob_type == scalar_types.uint32
-            || ob_type == scalar_types.uint16
-            || ob_type == scalar_types.uint8
-            || ob_type == scalar_types.bool_
-            || ob_type == scalar_types.datetime64
-    }
-}
-
-pub fn is_numpy_array(ob_type: *mut PyTypeObject) -> bool {
-    let numpy_types = unsafe { NUMPY_TYPES.get_or_init(load_numpy_types) };
-    if numpy_types.is_none() {
-        false
-    } else {
-        let scalar_types = unsafe { numpy_types.unwrap().as_ref() };
-        unsafe { ob_type == scalar_types.array }
-    }
-}
-
 #[repr(C)]
 pub struct PyCapsule {
     pub ob_base: PyObject,
@@ -53,7 +21,7 @@ pub struct PyCapsule {
     pub destructor: *mut c_void, // should be typedef void (*PyCapsule_Destructor)(PyObject *);
 }
 
-// https://docs.scipy.org/doc/numpy/reference/arrays.interface.html#c.__array_struct__
+// https://numpy.org/doc/1.26/reference/arrays.interface.html#object.__array_struct__
 
 #[repr(C)]
 pub struct PyArrayInterface {
@@ -69,7 +37,7 @@ pub struct PyArrayInterface {
 }
 
 #[derive(Clone, Copy)]
-pub enum ItemType {
+enum ItemType {
     BOOL,
     DATETIME64(NumpyDatetimeUnit),
     F16,
@@ -108,12 +76,139 @@ impl ItemType {
         }
     }
 }
+
 pub enum PyArrayError {
     Malformed,
     NotContiguous,
     UnsupportedDataType,
 }
 
+struct NumpyArrayData {
+    data: *const c_void,
+    len: usize,
+    kind: ItemType,
+    opts: Opt,
+}
+
+impl Serialize for NumpyArrayData {
+    #[inline(never)]
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(Some(self.len)).unwrap();
+        match self.kind {
+            ItemType::BOOL => {
+                let slice: &[u8] = slice!(self.data as *const u8, self.len);
+                for &each in slice.iter() {
+                    let value = each == 1;
+                    seq.serialize_element(&value).unwrap();
+                }
+            }
+            ItemType::DATETIME64(unit) => {
+                let slice: &[i64] = slice!(self.data as *const i64, self.len);
+                for &each in slice.iter() {
+                    let value = unit
+                        .datetime(each, self.opts)
+                        .map_err(serde::ser::Error::custom)?;
+                    seq.serialize_element(&value).unwrap();
+                }
+            }
+            ItemType::F16 => {
+                let slice: &[u16] = slice!(self.data as *const u16, self.len);
+                for &each in slice.iter() {
+                    let value = half::f16::from_bits(each).to_f32();
+                    seq.serialize_element(&value).unwrap();
+                }
+            }
+            ItemType::F32 => {
+                let slice: &[f32] = slice!(self.data as *const f32, self.len);
+                for &each in slice.iter() {
+                    seq.serialize_element(&each).unwrap();
+                }
+            }
+            ItemType::F64 => {
+                let slice: &[f64] = slice!(self.data as *const f64, self.len);
+                for &each in slice.iter() {
+                    seq.serialize_element(&each).unwrap();
+                }
+            }
+            ItemType::I8 => {
+                let slice: &[i8] = slice!(self.data as *const i8, self.len);
+                for &each in slice.iter() {
+                    seq.serialize_element(&each).unwrap();
+                }
+            }
+            ItemType::I16 => {
+                let slice: &[i16] = slice!(self.data as *const i16, self.len);
+                for &each in slice.iter() {
+                    seq.serialize_element(&each).unwrap();
+                }
+            }
+            ItemType::I32 => {
+                let slice: &[i32] = slice!(self.data as *const i32, self.len);
+                for &each in slice.iter() {
+                    seq.serialize_element(&each).unwrap();
+                }
+            }
+            ItemType::I64 => {
+                let slice: &[i64] = slice!(self.data as *const i64, self.len);
+                for &each in slice.iter() {
+                    seq.serialize_element(&each).unwrap();
+                }
+            }
+            ItemType::U8 => {
+                let slice: &[u8] = slice!(self.data as *const u8, self.len);
+                for &each in slice.iter() {
+                    seq.serialize_element(&each).unwrap();
+                }
+            }
+            ItemType::U16 => {
+                let slice: &[u16] = slice!(self.data as *const u16, self.len);
+                for &each in slice.iter() {
+                    seq.serialize_element(&each).unwrap();
+                }
+            }
+            ItemType::U32 => {
+                let slice: &[u32] = slice!(self.data as *const u32, self.len);
+                for &each in slice.iter() {
+                    seq.serialize_element(&each).unwrap();
+                }
+            }
+            ItemType::U64 => {
+                let slice: &[u64] = slice!(self.data as *const u64, self.len);
+                for &each in slice.iter() {
+                    seq.serialize_element(&each).unwrap();
+                }
+            }
+        }
+        seq.end()
+    }
+}
+
+enum NumpyArrayNode {
+    Internal(Vec<NumpyArrayNode>),
+    Leaf(NumpyArrayData),
+}
+
+impl Serialize for NumpyArrayNode {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        match self {
+            Self::Internal(children) => {
+                let mut seq = serializer.serialize_seq(Some(children.len())).unwrap();
+                for child in children {
+                    seq.serialize_element(child).unwrap();
+                }
+                seq.end()
+            }
+            Self::Leaf(value) => value.serialize(serializer),
+        }
+    }
+}
+
 // >>> arr = numpy.array([[[1, 2], [3, 4]], [[5, 6], [7, 8]]], numpy.int32)
 // >>> arr.ndim
 // 3
@@ -122,13 +217,8 @@ pub enum PyArrayError {
 // >>> arr.strides
 // (16, 8, 4)
 pub struct NumpyArray {
-    array: *mut PyArrayInterface,
-    position: Vec<isize>,
-    children: Vec<NumpyArray>,
-    depth: usize,
-    capsule: *mut PyCapsule,
-    kind: ItemType,
-    opts: Opt,
+    capsule: *mut PyObject,
+    root: NumpyArrayNode,
 }
 
 impl NumpyArray {
@@ -154,601 +244,76 @@ impl NumpyArray {
                     Err(PyArrayError::UnsupportedDataType)
                 }
                 Some(kind) => {
-                    let mut pyarray = NumpyArray {
-                        array: array,
-                        position: vec![0; num_dimensions],
-                        children: Vec::with_capacity(num_dimensions),
-                        depth: 0,
-                        capsule: capsule as *mut PyCapsule,
-                        kind: kind,
-                        opts: opts,
+                    let root = if num_dimensions > 1 {
+                        let mut position = Vec::with_capacity(num_dimensions);
+                        NumpyArray::build(capsule as *mut PyCapsule, kind, opts, 0, &mut position)
+                    } else {
+                        let shape = slice!((*array).shape as *const isize, num_dimensions);
+                        NumpyArrayNode::Leaf(NumpyArrayData {
+                            data: unsafe { (*array).data },
+                            len: shape[0] as usize,
+                            kind: kind,
+                            opts: opts,
+                        })
                     };
-                    if pyarray.dimensions() > 1 {
-                        pyarray.build();
-                    }
-                    Ok(pyarray)
+                    Ok(NumpyArray {
+                        capsule: capsule,
+                        root: root,
+                    })
                 }
             }
         }
     }
 
-    fn to_children(&self, position: Vec<isize>, num_children: usize) -> Self {
-        let mut arr = NumpyArray {
-            array: self.array,
-            position: position,
-            children: Vec::with_capacity(num_children),
-            depth: self.depth + 1,
-            capsule: self.capsule,
-            kind: self.kind,
-            opts: self.opts,
-        };
-        arr.build();
-        arr
-    }
-
-    fn build(&mut self) {
-        if self.depth < self.dimensions() - 1 {
-            for i in 0..=self.shape()[self.depth] - 1 {
-                let mut position: Vec<isize> = self.position.to_vec();
-                position[self.depth] = i;
-                let num_children: usize = if self.depth < self.dimensions() - 2 {
-                    self.shape()[self.depth + 1] as usize
-                } else {
-                    0
-                };
-                self.children.push(self.to_children(position, num_children))
-            }
+    fn build(
+        capsule: *mut PyCapsule,
+        kind: ItemType,
+        opts: Opt,
+        depth: usize,
+        position: &mut Vec<isize>,
+    ) -> NumpyArrayNode {
+        let array = unsafe { (*capsule).pointer as *mut PyArrayInterface };
+        let num_dimensions = unsafe { (*array).nd as usize };
+        let shape = slice!((*array).shape as *const isize, num_dimensions);
+        let strides = slice!((*array).strides as *const isize, num_dimensions);
+        let num_children = shape[depth];
+        let mut children = Vec::with_capacity(num_children as usize);
+        for i in 0..num_children {
+            position.push(i);
+            let child = if depth < num_dimensions - 2 {
+                NumpyArray::build(capsule, kind, opts, depth + 1, position)
+            } else {
+                let offset = strides
+                    .iter()
+                    .zip(position.iter())
+                    .map(|(a, b)| a * b)
+                    .sum::<isize>();
+                NumpyArrayNode::Leaf(NumpyArrayData {
+                    data: unsafe { (*array).data.offset(offset) },
+                    len: shape[num_dimensions - 1] as usize,
+                    kind: kind,
+                    opts: opts,
+                })
+            };
+            position.pop();
+            children.push(child);
         }
-    }
-
-    fn data(&self) -> *const c_void {
-        let offset = self
-            .strides()
-            .iter()
-            .zip(self.position.iter().copied())
-            .take(self.depth)
-            .map(|(a, b)| a * b)
-            .sum::<isize>();
-        unsafe { (*self.array).data.offset(offset) }
-    }
-
-    fn num_items(&self) -> usize {
-        self.shape()[self.shape().len() - 1] as usize
-    }
-
-    fn dimensions(&self) -> usize {
-        unsafe { (*self.array).nd as usize }
-    }
-
-    fn shape(&self) -> &[isize] {
-        slice!((*self.array).shape as *const isize, self.dimensions())
-    }
-
-    fn strides(&self) -> &[isize] {
-        slice!((*self.array).strides as *const isize, self.dimensions())
+        NumpyArrayNode::Internal(children)
     }
 }
 
 impl Drop for NumpyArray {
     fn drop(&mut self) {
-        if self.depth == 0 {
-            ffi!(Py_DECREF(self.array as *mut PyObject));
-            ffi!(Py_DECREF(self.capsule as *mut PyObject));
-        }
+        ffi!(Py_DECREF(self.capsule));
     }
 }
 
 impl Serialize for NumpyArray {
-    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        if !(self.depth >= self.shape().len() || self.shape()[self.depth] != 0) {
-            serializer.serialize_seq(Some(0)).unwrap().end()
-        } else if !self.children.is_empty() {
-            let mut seq = serializer.serialize_seq(Some(self.children.len())).unwrap();
-            for child in &self.children {
-                seq.serialize_element(child).unwrap();
-            }
-            seq.end()
-        } else {
-            let data_ptr = self.data();
-            let num_items = self.num_items();
-            let mut seq = serializer.serialize_seq(Some(num_items)).unwrap();
-            match self.kind {
-                ItemType::F64 => {
-                    let slice: &[f64] = slice!(data_ptr as *const f64, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeF64 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::F32 => {
-                    let slice: &[f32] = slice!(data_ptr as *const f32, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeF32 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::F16 => {
-                    let slice: &[u16] = slice!(data_ptr as *const u16, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeF16 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::U64 => {
-                    let slice: &[u64] = slice!(data_ptr as *const u64, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeU64 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::U32 => {
-                    let slice: &[u32] = slice!(data_ptr as *const u32, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeU32 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::U16 => {
-                    let slice: &[u16] = slice!(data_ptr as *const u16, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeU16 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::U8 => {
-                    let slice: &[u8] = slice!(data_ptr as *const u8, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeU8 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::I64 => {
-                    let slice: &[i64] = slice!(data_ptr as *const i64, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeI64 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::I32 => {
-                    let slice: &[i32] = slice!(data_ptr as *const i32, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeI32 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::I16 => {
-                    let slice: &[i16] = slice!(data_ptr as *const i16, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeI16 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::I8 => {
-                    let slice: &[i8] = slice!(data_ptr as *const i8, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeI8 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::BOOL => {
-                    let slice: &[u8] = slice!(data_ptr as *const u8, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeBool { obj: each }).unwrap();
-                    }
-                }
-                ItemType::DATETIME64(unit) => {
-                    let slice: &[i64] = slice!(data_ptr as *const i64, num_items);
-                    for &each in slice.iter() {
-                        let dt = unit
-                            .datetime(each, self.opts)
-                            .map_err(serde::ser::Error::custom)?;
-                        seq.serialize_element(&dt).unwrap();
-                    }
-                }
-            }
-            seq.end()
-        }
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeF64 {
-    pub obj: f64,
-}
-
-impl Serialize for DataTypeF64 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_f64(self.obj)
-    }
-}
-
-#[repr(transparent)]
-struct DataTypeF32 {
-    pub obj: f32,
-}
-
-impl Serialize for DataTypeF32 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_f32(self.obj)
-    }
-}
-
-#[repr(transparent)]
-struct DataTypeF16 {
-    pub obj: u16,
-}
-
-impl Serialize for DataTypeF16 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        let value = half::f16::from_bits(self.obj);
-        serializer.serialize_f32(value.to_f32())
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeU64 {
-    pub obj: u64,
-}
-
-impl Serialize for DataTypeU64 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u64(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeU32 {
-    pub obj: u32,
-}
-
-impl Serialize for DataTypeU32 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u32(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeU16 {
-    pub obj: u16,
-}
-
-impl Serialize for DataTypeU16 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u16(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeU8 {
-    pub obj: u8,
-}
-
-impl Serialize for DataTypeU8 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u8(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeI64 {
-    pub obj: i64,
-}
-
-impl Serialize for DataTypeI64 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i64(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeI32 {
-    pub obj: i32,
-}
-
-impl Serialize for DataTypeI32 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i32(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeI16 {
-    pub obj: i16,
-}
-
-impl Serialize for DataTypeI16 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i16(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeI8 {
-    pub obj: i8,
-}
-
-impl Serialize for DataTypeI8 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i8(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeBool {
-    pub obj: u8,
-}
-
-impl Serialize for DataTypeBool {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_bool(self.obj == 1)
-    }
-}
-
-pub struct NumpyScalar {
-    pub ptr: *mut PyObject,
-    opts: Opt,
-}
-
-impl NumpyScalar {
-    pub fn new(ptr: *mut PyObject, opts: Opt) -> Self {
-        NumpyScalar { ptr, opts }
-    }
-}
-
-impl Serialize for NumpyScalar {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        unsafe {
-            let ob_type = ob_type!(self.ptr);
-            let scalar_types =
-                unsafe { NUMPY_TYPES.get_or_init(load_numpy_types).unwrap().as_ref() };
-            if ob_type == scalar_types.float64 {
-                (*(self.ptr as *mut NumpyFloat64)).serialize(serializer)
-            } else if ob_type == scalar_types.float32 {
-                (*(self.ptr as *mut NumpyFloat32)).serialize(serializer)
-            } else if ob_type == scalar_types.float16 {
-                (*(self.ptr as *mut NumpyFloat16)).serialize(serializer)
-            } else if ob_type == scalar_types.int64 {
-                (*(self.ptr as *mut NumpyInt64)).serialize(serializer)
-            } else if ob_type == scalar_types.int32 {
-                (*(self.ptr as *mut NumpyInt32)).serialize(serializer)
-            } else if ob_type == scalar_types.int16 {
-                (*(self.ptr as *mut NumpyInt16)).serialize(serializer)
-            } else if ob_type == scalar_types.int8 {
-                (*(self.ptr as *mut NumpyInt8)).serialize(serializer)
-            } else if ob_type == scalar_types.uint64 {
-                (*(self.ptr as *mut NumpyUint64)).serialize(serializer)
-            } else if ob_type == scalar_types.uint32 {
-                (*(self.ptr as *mut NumpyUint32)).serialize(serializer)
-            } else if ob_type == scalar_types.uint16 {
-                (*(self.ptr as *mut NumpyUint16)).serialize(serializer)
-            } else if ob_type == scalar_types.uint8 {
-                (*(self.ptr as *mut NumpyUint8)).serialize(serializer)
-            } else if ob_type == scalar_types.bool_ {
-                (*(self.ptr as *mut NumpyBool)).serialize(serializer)
-            } else if ob_type == scalar_types.datetime64 {
-                let unit = NumpyDatetimeUnit::from_pyobject(self.ptr);
-                let obj = &*(self.ptr as *mut NumpyDatetime64);
-                let dt = unit
-                    .datetime(obj.value, self.opts)
-                    .map_err(serde::ser::Error::custom)?;
-                dt.serialize(serializer)
-            } else {
-                unreachable!()
-            }
-        }
-    }
-}
-
-#[repr(C)]
-pub struct NumpyInt8 {
-    pub ob_base: PyObject,
-    pub value: i8,
-}
-
-impl Serialize for NumpyInt8 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i8(self.value)
-    }
-}
-
-#[repr(C)]
-pub struct NumpyInt16 {
-    pub ob_base: PyObject,
-    pub value: i16,
-}
-
-impl Serialize for NumpyInt16 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i16(self.value)
-    }
-}
-
-#[repr(C)]
-pub struct NumpyInt32 {
-    pub ob_base: PyObject,
-    pub value: i32,
-}
-
-impl Serialize for NumpyInt32 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i32(self.value)
-    }
-}
-
-#[repr(C)]
-pub struct NumpyInt64 {
-    pub ob_base: PyObject,
-    pub value: i64,
-}
-
-impl Serialize for NumpyInt64 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i64(self.value)
-    }
-}
-
-#[repr(C)]
-pub struct NumpyUint8 {
-    pub ob_base: PyObject,
-    pub value: u8,
-}
-
-impl Serialize for NumpyUint8 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u8(self.value)
-    }
-}
-
-#[repr(C)]
-pub struct NumpyUint16 {
-    pub ob_base: PyObject,
-    pub value: u16,
-}
-
-impl Serialize for NumpyUint16 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u16(self.value)
-    }
-}
-
-#[repr(C)]
-pub struct NumpyUint32 {
-    pub ob_base: PyObject,
-    pub value: u32,
-}
-
-impl Serialize for NumpyUint32 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u32(self.value)
-    }
-}
-
-#[repr(C)]
-pub struct NumpyUint64 {
-    pub ob_base: PyObject,
-    pub value: u64,
-}
-
-impl Serialize for NumpyUint64 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u64(self.value)
-    }
-}
-
-#[repr(C)]
-pub struct NumpyFloat16 {
-    pub ob_base: PyObject,
-    pub value: u16,
-}
-
-impl Serialize for NumpyFloat16 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        let value = half::f16::from_bits(self.value);
-        serializer.serialize_f32(value.to_f32())
-    }
-}
-
-#[repr(C)]
-pub struct NumpyFloat32 {
-    pub ob_base: PyObject,
-    pub value: f32,
-}
-
-impl Serialize for NumpyFloat32 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_f32(self.value)
-    }
-}
-
-#[repr(C)]
-pub struct NumpyFloat64 {
-    pub ob_base: PyObject,
-    pub value: f64,
-}
-
-impl Serialize for NumpyFloat64 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_f64(self.value)
-    }
-}
-
-#[repr(C)]
-pub struct NumpyBool {
-    pub ob_base: PyObject,
-    pub value: bool,
-}
-
-impl Serialize for NumpyBool {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_bool(self.value)
+        self.root.serialize(serializer)
     }
 }
 
@@ -757,7 +322,7 @@ impl Serialize for NumpyBool {
 /// See
 /// https://github.com/numpy/numpy/blob/v1.26.4/numpy/core/include/numpy/ndarraytypes.h#L244-L258
 #[derive(Clone, Copy)]
-pub enum NumpyDatetimeUnit {
+enum NumpyDatetimeUnit {
     NaT,
     Years,
     Months,
@@ -925,8 +490,102 @@ impl NumpyDatetimeUnit {
     }
 }
 
+macro_rules! define_numpy_type {
+    ($name:ident, $object_name:ident, $type:ty) => {
+        #[repr(C)]
+        struct $object_name {
+            ob_base: PyObject,
+            value: $type,
+        }
+
+        #[repr(transparent)]
+        pub struct $name {
+            ptr: *mut PyObject,
+        }
+
+        impl $name {
+            pub fn new(ptr: *mut PyObject) -> Self {
+                $name { ptr }
+            }
+        }
+
+        impl Serialize for $name {
+            fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+            where
+                S: Serializer,
+            {
+                let value = unsafe { (*(self.ptr as *mut $object_name)).value };
+                value.serialize(serializer)
+            }
+        }
+    };
+}
+
+define_numpy_type!(NumpyBool, NumpyBoolObject, bool);
+define_numpy_type!(NumpyFloat32, NumpyFloat32Object, f32);
+define_numpy_type!(NumpyFloat64, NumpyFloat64Object, f64);
+define_numpy_type!(NumpyInt8, NumpyInt8Object, i8);
+define_numpy_type!(NumpyInt16, NumpyInt16Object, i16);
+define_numpy_type!(NumpyInt32, NumpyInt32Object, i32);
+define_numpy_type!(NumpyInt64, NumpyInt64Object, i64);
+define_numpy_type!(NumpyUint8, NumpyUint8Object, u8);
+define_numpy_type!(NumpyUint16, NumpyUint16Object, u16);
+define_numpy_type!(NumpyUint32, NumpyUint32Object, u32);
+define_numpy_type!(NumpyUint64, NumpyUint64Object, u64);
+
 #[repr(C)]
-pub struct NumpyDatetime64 {
-    pub ob_base: PyObject,
+struct NumpyDatetime64Object {
+    ob_base: PyObject,
     value: i64,
 }
+
+pub struct NumpyDatetime64 {
+    ptr: *mut PyObject,
+    opts: Opt,
+}
+
+impl NumpyDatetime64 {
+    pub fn new(ptr: *mut PyObject, opts: Opt) -> Self {
+        NumpyDatetime64 { ptr, opts }
+    }
+}
+
+impl Serialize for NumpyDatetime64 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let unit = NumpyDatetimeUnit::from_pyobject(self.ptr);
+        let value = unsafe { (*(self.ptr as *mut NumpyDatetime64Object)).value };
+        unit.datetime(value, self.opts)
+            .map_err(serde::ser::Error::custom)?
+            .serialize(serializer)
+    }
+}
+
+#[repr(C)]
+struct NumpyFloat16Object {
+    ob_base: PyObject,
+    value: u16,
+}
+
+#[repr(transparent)]
+pub struct NumpyFloat16 {
+    ptr: *mut PyObject,
+}
+
+impl NumpyFloat16 {
+    pub fn new(ptr: *mut PyObject) -> Self {
+        NumpyFloat16 { ptr }
+    }
+}
+
+impl Serialize for NumpyFloat16 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let value = unsafe { (*(self.ptr as *mut NumpyFloat16Object)).value };
+        half::f16::from_bits(value).to_f32().serialize(serializer)
+    }
+}
